struct VERT_INPUT
{
	float4 location : POSITION;
	float4 color : COLOR0;
	float2 uv : TEXCOORD0;
};

struct VERT_ANIMATION_INFO
{
	float2 uv : TEXCOORD0;
	float horizontalUVOffsetPerFrame : NORMAL0;
	float animationInterval : NORMAL1;
	float animationStartTime : NORMAL2;
	float animationFrames : NORMAL3;
};

struct VERT_INPUT_ATLAS
{
	float4 location : POSITION;
	float4 color : COLOR0;
	float2 modUV : TEXCOORD1;
	VERT_ANIMATION_INFO animInfo;
	float modFactor : NORMAL4;
};

struct VERT_OUTPUT
{
	float4 location : POSITION;
	float4 color : COLOR0;
	float2 uv : TEXCOORD0;
};

struct VERT_OUTPUT_ATLAS
{
	float4 location : POSITION;
	float4 color : COLOR0;
	float2 uv : TEXCOORD0;
	float2 modUV : TEXCOORD1;
	float modFactor : NORMAL0;
};

struct PIX_INPUT
{
	float4 color : COLOR0;
	float2 uv : TEXCOORD0;
};

struct PIX_INPUT_ATLAS
{
	float4 color : COLOR0;
	float2 uv : TEXCOORD0;
	float2 modUV : TEXCOORD1;
	float modFactor : NORMAL0;
};

typedef float4 PIX_OUTPUT;

static const float PI = 3.14159265f;

float4x4 _transform;
float _time;
float _gameTime;

sampler2D _texture;
float4 _color;

float2 getAnimatedUVsLooping(VERT_ANIMATION_INFO animInfo)
{
	float2 ret;
	ret.y = animInfo.uv.y;
	int frame = (int)((_gameTime - animInfo.animationStartTime) / animInfo.animationInterval) % (int)animInfo.animationFrames;
	ret.x = animInfo.uv.x + animInfo.horizontalUVOffsetPerFrame * frame;
	return ret;
}

float2 getAnimatedUVsClamped(VERT_ANIMATION_INFO animInfo)
{
	float2 ret;
	ret.y = animInfo.uv.y;
	int frame = min((int)((_gameTime - animInfo.animationStartTime) / animInfo.animationInterval), (int)animInfo.animationFrames - 1);
	ret.x = animInfo.uv.x + animInfo.horizontalUVOffsetPerFrame * frame;
	return ret;
}

#ifdef USE_DEFAULT_VERT
VERT_OUTPUT vert(in VERT_INPUT input)
{
	VERT_OUTPUT output;
	output.location = mul(input.location, _transform);
	output.color = input.color;
	output.uv = input.uv;
	return output;
}
#endif

#ifdef USE_DEFAULT_VERT_ATLAS
VERT_OUTPUT_ATLAS vert(in VERT_INPUT_ATLAS input)
{
	VERT_OUTPUT_ATLAS output;
	output.location = mul(input.location, _transform);
	output.color = input.color;
	output.uv = getAnimatedUVsLooping(input.animInfo);
	output.modUV = input.modUV;
	output.modFactor = input.modFactor;
	return output;
}
#endif

#ifdef USE_CLAMPED_VERT_ATLAS
VERT_OUTPUT_ATLAS vert(in VERT_INPUT_ATLAS input)
{
	VERT_OUTPUT_ATLAS output;
	output.location = mul(input.location, _transform);
	output.color = input.color;
	output.uv = getAnimatedUVsClamped(input.animInfo);
	output.modUV = input.modUV;
	output.modFactor = input.modFactor;
	return output;
}
#endif

float4 getModulatedColor(float2 uv, float2 modUV, float modFactor)
{
	float4 baseColor = tex2D(_texture, uv);
	float4 modColor = tex2D(_texture, modUV);
	float4 modulated = baseColor * modColor;
	return baseColor * (1 - modFactor) + modulated * modFactor;
}

float4 getModulatedColor(PIX_INPUT_ATLAS input)
{
	return getModulatedColor(input.uv, input.modUV, input.modFactor);
}

float4 swapAllegianceColor(float4 pixel, float4 allegianceColor)
{
	if(pixel.g > pixel.r && abs(pixel.r - pixel.b) < .025) // Replace green pixels with allegiance color.
	{
		float3 scaledAllegianceColor = allegianceColor.rgb * pixel.g;
		float desaturation = pixel.r / pixel.g; // works because pixel.r == pixel.b (approximately)
		float desaturationValue = max(scaledAllegianceColor.r, max(scaledAllegianceColor.g, scaledAllegianceColor.b));
		float3 desaturationColor = float3(desaturationValue, desaturationValue, desaturationValue);
		pixel.rgb = scaledAllegianceColor * (1 - desaturation) + desaturationColor * desaturation;
	}
	return pixel;
}

#ifdef USE_DEFAULT_PIX
PIX_OUTPUT pix(in PIX_INPUT input) : COLOR0
{
	return tex2D(_texture, input.uv) * input.color * _color;
}
#endif

#ifdef USE_DEFAULT_PIX_ATLAS
PIX_OUTPUT pix(in PIX_INPUT_ATLAS input) : COLOR0
{
	return getModulatedColor(input) * input.color * _color;
}
#endif